

class BaseHandler():
    """
    BaseHandler class is used to load configuration specific to the each handler
    """
    def __init__(self, LogLevel, LogFormat, UseLogLineStartTag):
        self._logLevel = LogLevel
        self._logFormat = LogFormat
        self._useLogLineStartTag = UseLogLineStartTag


    @property
    def logLevel(self):
        return self._logLevel
    
    @property
    def logFormat(self):
        return self._logFormat

    @property
    def useLogLineStartTag(self):
        return self._useLogLineStartTag

    @property
    def settings(self):
        return {
            'logLevel': self._logLevel,
            'logFormat': self._logFormat,
            'useLogLineStartTag': self._useLogLineStartTag,
        }


from .logging import _SLogger as SLogger

from .paths import CONFIG_FILEPATH

from .config_utilities.yaml_utils import create_default_configuration_file


if not CONFIG_FILEPATH.is_file():
    create_default_configuration_file()




import logging

# internal imports
from ..config_utilities import Config
from .handlers.file_handler import FileHandler
from .handlers.console_handler import ConsoleHandler


class _SLogger(logging.getLoggerClass()):

    def __init__(self, name, logfilepath, level=logging.NOTSET) -> None:
        super().__init__(name, level)
        self.logfilepath = str(logfilepath)
        self._config = Config()
        self._assign_config_to_logger()

        self._consoleHandler = ConsoleHandler(self._consoleHandler_logLevel, self._consoleHandler_logFormat, self._consoleHandler_useLogLineStartTag)
        self._fileHandler = FileHandler(
            self._fileHandler_logLevel, self._fileHandler_logFormat, self._fileHandler_useLogLineStartTag,
            filepath = self._logFile,
        )

        self.setLevel(self._logLevel)
    
        if self._useFileHandler:
            self.addHandler(self._get_file_handler())
        if self._useConsoleHandler:
            self.addHandler(self._get_stream_handler())

        return
    
    @property
    def config(self):
        return self._config

    @property
    def logger(self):
        return self._logger

    @property
    def consoleHandler(self):
        return self._consoleHandler

    @property
    def fileHandler(self):
        return self._fileHandler
    

    def set_logfilepath(self, path):
        self.logfilepath = str(path)

        

    def _assign_config_to_logger(self) -> None:
        '''
        Assigns the configuration to the logger
        '''
        self._logFile = self.logfilepath
        self._logLevel = self._config.settings['logLevel']
        # print(self._logLevel)
        # self._logFile = self._config.settings['logFile']
        self._logFormat = self._config.settings['logFormat']
        self._useFileHandler = self._config.settings['useFileHandler']
        self._useConsoleHandler = self._config.settings['useConsoleHandler']
        self._useLogLineStartTag = self._config.settings['useLogLineStartTag']
        
        self._fileHandler_logLevel = self._config.settings['fileHandler']['logLevel']
        self._fileHandler_logFormat = self._config.settings['fileHandler']['logFormat']
        self._fileHandler_useLogLineStartTag = self._config.settings['fileHandler']['useLogLineStartTag']

        self._consoleHandler_logLevel = self._config.settings['consoleHandler']['logLevel']
        self._consoleHandler_logFormat = self._config.settings['consoleHandler']['logFormat']
        self._consoleHandler_useLogLineStartTag = self._config.settings['consoleHandler']['useLogLineStartTag']

        return

    def _get_file_handler(self):
        file_handler = logging.FileHandler(self._logFile)
        file_handler.setLevel(self._fileHandler_logLevel)
        file_handler.setFormatter(logging.Formatter(self._fileHandler_logFormat))
        return file_handler
    
    def _get_stream_handler(self):
        stream_handler = logging.StreamHandler()
        stream_handler.setLevel(self._consoleHandler_logLevel)
        stream_handler.setFormatter(logging.Formatter(self._consoleHandler_logFormat))
        return stream_handler












from . import yaml_utils as yaml
from ..paths import CONFIG_FILEPATH


class Config:

    def __init__(self, file_path=CONFIG_FILEPATH) -> None:
        self._config_filepath = file_path
        # self.create_default_yaml_configuration_file()
        self._config = self.load_from_yaml(self._config_filepath)
        return

    def load_from_yaml(self, file_path) -> dict:
        '''
        Loads the configuration from the yaml file
        '''

        config = yaml.load_configuration_file(file_path)
        config = yaml.prepare_config_settings(config) # here regenerating config happens
        return config


    def create_default_yaml_configuration_file(self) -> None:
        '''
        Creates a default configuration file for the application
        '''
        yaml.create_default_configuration_file()
        return
    
    @property
    def settings(self) -> dict:
        return self._config



from .base_handler import BaseHandler

class ConsoleHandler(BaseHandler):
    """
    ConsoleHandler class is used to load configuration specific to the cpnsole handler
    """
    def __init__(self, LogLevel, LogFormat, UseLogLineStartTag):
        super().__init__(LogLevel, LogFormat, UseLogLineStartTag)
        return





import os
import yaml  # PyYAML==5.3.1 is currently used

# internal imports
from ..paths import CONFIG_DIR, CONFIG_FILE, CONFIG_FILEPATH
from ..config import LOG_LINE_TAG

'''
YAML FILE RELATED FUNCTIONS
'''


def load_configuration_file(file_path) -> dict:
    """
    Load the configuration file
    :return: config: dict
    """
    with open(file_path) as f:
        config = yaml.safe_load(f)
        # print(config)
    return config


def prepare_config_settings(config:dict) -> dict:
    """
    Prepare the configuration settings to include the default values if not specified
    :param config: dict
    :return config: dict
    """

    config = load_configuration_file(CONFIG_FILEPATH)
    edited_config = {}

    edited_config['logLevel'] = config['logLevel']
    if config['useLogLineStartTag'] is True:
        edited_config['logFormat'] = f'''{LOG_LINE_TAG} {config['logFormat']}''' 
    else:
        edited_config['logFormat'] = config['logFormat']
    edited_config['useLogLineStartTag'] = config['useLogLineStartTag']

    edited_config['useFileHandler'] = config['useFileHandler']
    edited_config['useConsoleHandler'] = config['useConsoleHandler']

    edited_config['fileHandler'] = {}
    edited_config['consoleHandler'] = {}

    if config['useFileHandler'] is True:
        
        # configure_file_handler_configurations(config, edited_config)
        # FILE LOG LEVEL
        if config['fileHandler']['logLevel'] == "DEFAULT":
            edited_config['fileHandler']['logLevel'] = config['logLevel']
        else:
            edited_config['fileHandler']['logLevel'] = config['fileHandler']['logLevel']
        
        # FILE LOG FORMAT
        if config['fileHandler']['logFormat'] == "DEFAULT":
            edited_config['fileHandler']['logFormat'] = config['logFormat']
        else:
            edited_config['fileHandler']['logFormat'] = config['fileHandler']['logFormat']
        
        # FILE LOG LINE TAG
        if config['fileHandler']['useLogLineStartTag'] == "DEFAULT":
            edited_config['fileHandler']['useLogLineStartTag'] = config['useLogLineStartTag']
        else:
            edited_config['fileHandler']['useLogLineStartTag'] = config['fileHandler']['useLogLineStartTag']

        # FILE LOG LINE TAG - FORMATING -- LOGLINES
        if edited_config['fileHandler']['useLogLineStartTag'] is True:
            edited_config['fileHandler']['logFormat'] =f'''{LOG_LINE_TAG} {edited_config['fileHandler']['logFormat']}'''
        elif edited_config['fileHandler']['useLogLineStartTag'] is False:
            edited_config['fileHandler']['logFormat'] = edited_config['fileHandler']['logFormat']

    else:
        edited_config['fileHandler'] = config['fileHandler']

    # IF CONSOLE HANDLER IS ENABLED ===========================================================================
    if config['useConsoleHandler'] is True:
        
        # configure_file_handler_configurations(config, edited_config)
        # FILE LOG LEVEL
        if config['consoleHandler']['logLevel'] == "DEFAULT":
            edited_config['consoleHandler']['logLevel'] = config['logLevel']
        else:
            edited_config['consoleHandler']['logLevel'] = config['consoleHandler']['logLevel']
        
        # FILE LOG FORMAT
        if config['consoleHandler']['logFormat'] == "DEFAULT":
            edited_config['consoleHandler']['logFormat'] = config['logFormat']
        else:
            edited_config['consoleHandler']['logFormat'] = config['consoleHandler']['logFormat']
        
        # FILE LOG LINE TAG
        if config['consoleHandler']['useLogLineStartTag'] == "DEFAULT":
            edited_config['consoleHandler']['useLogLineStartTag'] = config['useLogLineStartTag']
        else:
            edited_config['consoleHandler']['useLogLineStartTag'] = config['consoleHandler']['useLogLineStartTag']

        # FILE LOG LINE TAG - FORMATING -- LOGLINES
        if edited_config['consoleHandler']['useLogLineStartTag'] is True:
            edited_config['consoleHandler']['logFormat'] =f'''{LOG_LINE_TAG} {edited_config['consoleHandler']['logFormat']}'''
        elif edited_config['consoleHandler']['useLogLineStartTag'] is False:
            edited_config['consoleHandler']['logFormat'] = edited_config['consoleHandler']['logFormat']

    else:
        edited_config['consoleHandler'] = config['consoleHandler']
    return edited_config

def create_default_configuration_file():
    """
    Create a default configuration file for the application
    :return:
    """

    config = dict(
        logLevel = 'DEBUG',
        # logFile = 'logs.log',
        logFormat = '''%(asctime)s | %(name)s | %(levelname)s (%(filename)s).%(funcName)s(%(lineno)d) - %(message)s''',
        # handlers = ['fileHandler', 'consoleHandler'],
        useFileHandler = True,
        useConsoleHandler = True,
        useLogLineStartTag = True,

        fileHandler = dict(
            logLevel = 'DEFAULT', # this needed. if so it will take from general log level
            logFormat = 'DEFAULT',
            useLogLineStartTag = 'DEFAULT',
        ),

        consoleHandler = dict(
            logLevel = 'DEFAULT',
            logFormat = "DEFAULT",
            useLogLineStartTag = 'DEFAULT',
        ),

    )

    if not CONFIG_DIR.is_dir():
        CONFIG_DIR.mkdir()

    config_filepath = os.path.join(CONFIG_DIR, CONFIG_FILE)

    with open(config_filepath, 'w') as f:
        yaml.dump(config, f, default_flow_style=False, sort_keys=False, indent=2, line_break='',width=1000)
    
    return config_filepath




from .base_handler import BaseHandler

class FileHandler(BaseHandler):
    """
    FileHandler class is used to load configuration specific to the file handler
    """
    def __init__(self, LogLevel, LogFormat, UseLogLineStartTag, filepath):
        super().__init__(LogLevel, LogFormat, UseLogLineStartTag)

        self._filepath = filepath
        return
    
    @property
    def settings(self):
        return {
            'logLevel': self._logLevel,
            'logFormat': self._logFormat,
            'useLogLineStartTag': self._useLogLineStartTag,
            'filepath': self._filepath,
        }




from pathlib import Path

PACKAGE_DIR = Path(__file__).resolve().parent
MAIN_DIR = PACKAGE_DIR.parent

# other directories
CONFIG_DIR = MAIN_DIR / 'configuration'
CONFIG_FILE = 'slogger.yaml'
CONFIG_FILEPATH = CONFIG_DIR / CONFIG_FILE

TEMP_DIR = MAIN_DIR / ".temp"
TEMP_DIR.mkdir(exist_ok=True)
